@@ -15,7 +15,7 @@ function (data, x, y, ceilings=c("ols", "ce_fdh", "cr_fdh"),
           corner=NULL, flip.x=FALSE, flip.y=FALSE, scope=NULL,
           bottleneck.x='percentage.range', bottleneck.y='percentage.range',
           steps=10, step.size=NULL, cutoff=0, qr.tau=0.95, effect_aggregation=1,
-          test.rep=0, test.p_confidence=0.95, test.p_threshold=0) {
+          test.rep=0, test.p_confidence=0.95, test.p_threshold=0.05) {
 
   # Cleans up any cluster registration
   p_cluster_cleanup()
