@@ -131,3 +131,75 @@ function (bn, method, title) {
   print(tmp, quote=FALSE)
   message()
 }
+
+p_display_table_screen_tab <-
+function (bn, method, title) {
+  names <- colnames(bn)
+  bn.x <- attr(bn, "bn.x")
+  bn.y <- attr(bn, "bn.y")
+  bn.y.id <- attr(bn, "bn.y.id")
+  rows <- nrow(bn)
+  x.length <- ncol(bn) - 1
+  size <- attr(bn, "size")
+  cutoff <- attr(bn, "cutoff")
+
+  # TODO Weird bug : can't replace colnames in bn
+  tmp <- matrix(nrow=rows, ncol=x.length)
+  for (i in 1:x.length) {
+    if (bn.x == 'percentile') {
+      suppressWarnings(
+        cases <- round(size * as.numeric(bn[,i+1]) / 100, digits = 0)
+      )
+      tmp[, i] <- paste0(bn[,i+1], ' (', as.character(cases), ')')
+    } else {
+      tmp[, i] <- bn[,i+1]
+    }
+  }
+  if (length(tmp) == 0) {
+    return()
+  }
+
+  # Set y precision
+  if (bn.y.id %in% c(1, 2)) {
+    digits <- ifelse((100 / (rows-1)) %% 1 == 0, 0, 1)
+  } else {
+    digits <- p_get_digits(bn[,1])
+  }
+  colnames(tmp) <- as.character(1:x.length)
+  rownames(tmp) <- sapply(bn[,1], p_pretty_number, "", digits, TRUE)
+
+  # Display header
+  tab <- 8
+  tabs <- ceiling((3 + max(nchar(names))) / tab)
+  offset <- floor(log10(x.length)) + 2
+  fmt <- sprintf("%%%ds ", offset - 1)
+  cat("\n----------------------------------------")
+  cat("----------------------------------------\n")
+  message(title, " ", method, " (cutoff = ", cutoff, ")")
+  needed <- tabs - floor((offset + nchar(names[1])) / tab)
+  message(sprintf(fmt, "Y"), names[1], strrep("\t", needed), "(", bn.y, ")")
+  for (i in seq(x.length)) {
+    needed <- tabs - floor((offset + nchar(names[i + 1])) / tab)
+    message(sprintf(fmt, i),names[i + 1], strrep("\t", needed), "(", bn.x, ")")
+  }
+  cat("----------------------------------------")
+  cat("----------------------------------------\n")
+
+  # Display table
+  tabs <- ceiling(max(nchar(tmp)) / tab)
+  first <- ceiling(max(nchar(colnames(tmp))) / tab)
+  output <- c("Y", strrep("\t", first))
+  output <- c(output, paste0(colnames(tmp), strrep("\t", tabs)), "\n")
+  for (idx.row in 1:nrow(tmp)) {
+    needed <- first - floor(nchar(rownames(tmp)[idx.row]) / tab)
+    output <- c(output, rownames(tmp)[idx.row], strrep("\t", needed))
+    for (idx.col in 1:ncol(tmp)) {
+      needed <- tabs - floor(nchar(tmp[idx.row, idx.col]) / tab)
+      output <- c(output, tmp[idx.row, idx.col], strrep("\t", needed))
+    }
+    output <- c(output, "\n")
+  }
+  message(paste(output, collapse=''))
+
+  message()
+}
