@@ -1,7 +1,11 @@
+HIDDEN <- 'hidden'
+SHOWN <- 'shown'
+
 nca_outliers <- function (data, x, y, ceiling = NULL,
                           corner = NULL, flip.x = FALSE, flip.y = FALSE,
+                          scope = NULL,
                           k = 1, min.dif = 1e-2, max.results = 25,
-                          plotly = FALSE) {
+                          plotly = FALSE, condensed = FALSE) {
   # Cleans up any cluster registration
   p_cluster_cleanup()
 
@@ -12,23 +16,25 @@ nca_outliers <- function (data, x, y, ceiling = NULL,
   if (is.null(input.ok)) {
     ceiling <- "ce_fdh"
   }
+  data <- as.data.frame(p_validate_clean(data, x, y, TRUE))
+  x <- colnames(data)[1]
+  y <- colnames(data)[2]
 
-  model <- nca_analysis(data, x, y, ceilings = ceiling,
-                        corner = corner, flip.x = flip.x, flip.y = flip.y)
+  model <- nca_analysis(data, x, y, ceilings = ceiling, corner = corner,
+                        flip.x = flip.x, flip.y = flip.y, scope = scope)
   eff.or <- model$summaries[[1]]$params[2]
   global <- model$summaries[[1]]$global
   params <- list(model = model, x = x, y = y, ceiling = ceiling,
                  corner = corner, flip.x = flip.x, flip.y = flip.y,
-                 eff.or = eff.or, global = global,
-                 min.dif = min.dif,
-                 peers = model$peers[[1]])
+                 scope = scope, eff.or = eff.or, global = global,
+                 min.dif = min.dif, peers = model$peers[[1]])
 
   org_outliers <- p_get_outliers(data, params, 1)
-  outliers <- p_format_outliers(org_outliers, max.results, 1)
   if (is.null(org_outliers)) {
     message("\nNo outliers identified")
     return()
   }
+  outliers <- p_format_outliers(org_outliers, max.results, 1, min.dif, condensed)
 
   if (plotly) {
     points <- data[outliers[, 1], c(x, y)]
@@ -45,7 +51,7 @@ nca_outliers <- function (data, x, y, ceiling = NULL,
   }
 
   outliers <- p_get_outliers(data, params, k, org_outliers)
-  return(p_format_outliers(outliers, max.results, k))
+  return(p_format_outliers(outliers, max.results, k, min.dif, condensed))
 }
 
 
@@ -97,25 +103,21 @@ p_get_outliers <- function (data, params, k, org_outliers = NULL) {
 
   idx <- NULL
   ids <- seq(1, nrow(combos), max(1, round(nrow(combos) / 50)))
-  outliers <- foreach(idx = 1:nrow(combos), .combine = 'rbind') %dopar% {
+  outliers <- foreach(idx = 1:nrow(combos), .combine = rbind) %dopar% {
     if (condition && idx %in% ids) {
       cat(".")
     }
     return(p_get_outlier(data, combos[idx,], params, k))
   }
   if (condition) {
-    message("\rDone", strrep(' ', 50))
+    message("\rDone", strrep(' ', 50), "\n")
   }
 
-  if (condition) {
-    message("Finalizing the analysis")
+  if (is.null(outliers)) {
+    return (org_outliers)
   }
-  ids <- round(seq(1, nrow(outliers), nrow(outliers) / 50))
-  new_outliers <- foreach(idx = 1:nrow(outliers), .combine = rbind
-      ) %dopar% {
-    if (condition && idx %in% ids) {
-      cat(".")
-    }
+
+  new_outliers <- foreach(idx = 1:nrow(outliers), .combine = rbind) %dopar% {
     outlier <- outliers[idx, ]
     old_combo <- outlier[8]
     tmp <- org_outliers
@@ -126,18 +128,15 @@ p_get_outliers <- function (data, params, k, org_outliers = NULL) {
     #}
 
     f <- function (n) {
-      dif.rel <- tmp[tmp[, "combo"] %in% n, ]$dif.rel
-      return( abs(ifelse(is.null(dif.rel), 0, dif.rel)) )
+      dif.rel <- tmp[tmp[, "combo"] %in% n,]$dif.rel
+      return (abs(ifelse(is.null(dif.rel), 0, dif.rel)))
     }
-    ord <- unname(unlist(sapply(unlist(old_combo), f)))
-    new_combo <- old_combo[order(-ord)]
-    outlier[8] <- new_combo[!sapply(new_combo, is.null)]
-    outlier[1] <- p_get_names(unlist(outlier[8]), k)$outliers
 
-    return ( outlier )
-  }
-  if (condition) {
-    message("\rDone", strrep(' ', 50))
+    ord <- unlist(sapply(unlist(old_combo), f))
+    tmp <- list(unlist(old_combo)[order(-ord)])
+    outlier[1] <- p_get_names(unlist(tmp), k)$outliers
+
+    return (outlier)
   }
 
   stopImplicitCluster()
@@ -157,7 +156,7 @@ p_get_all_names <- function (data, peers, global, params, k) {
     }
   }
 
-  return( all.names )
+  return (all.names)
 }
 
 
@@ -172,9 +171,9 @@ p_get_combos <- function (data, params, k) {
   counter <- k
   while (counter > 1) {
     data <- data[!(row.names(data) %in% all.names),]
-    model <- nca_analysis(data, params$x, params$y,
-                          ceilings = params$ceiling, corner = params$corner,
-                          flip.x = params$flip.x, flip.y = params$flip.y)
+    model <- nca_analysis(data, params$x, params$y, ceilings = params$ceiling,
+                          corner = params$corner, flip.x = params$flip.x,
+                          flip.y = params$flip.y, scope = params$scope)
 
     global <- model$summaries[[1]]$global
     all.names <- c(
@@ -185,11 +184,20 @@ p_get_combos <- function (data, params, k) {
     counter <- counter - 1
   }
 
+
   all.names <- unique(all.names)
-  combos <- t(combn(all.names, k))
 
   if (k == 1) {
-    combos <- matrix(combos, ncol = 1)
+    return (matrix(all.names, ncol = 1))
+  }
+
+  combos <- NULL
+  counter <- k
+  while (counter > 1) {
+    tmp <- t(combn(all.names, counter))
+    tmp <- cbind(tmp, matrix(NA, ncol = k - counter, nrow = nrow(tmp)))
+    combos <- rbind(combos, tmp)
+    counter <- counter - 1
   }
 
   return(combos)
@@ -197,15 +205,17 @@ p_get_combos <- function (data, params, k) {
 
 
 p_get_outlier <- function (data, combo, params, k) {
+  combo <- combo[!is.na(combo)]
   data.new <- data[-(which(rownames(data) %in% combo)),]
   values <- p_get_values(data.new, params)
-  eff.nw <- values[1]; dif.abs <- values[2]; dif.rel <- values[3]
+  eff.nw <- values[[1]]; dif.abs <- values[[2]]; dif.rel <- values[[3]]
+  global.new <- values[[4]]
 
   if (round(abs(dif.rel), digits = 2) < params$min.dif) {
     return()
   }
 
-  zone_scope <- p_zone_scope(data, combo, params)
+  zone_scope <- p_zone_scope(combo, params, global.new)
   zone <- zone_scope[1]; scope <- zone_scope[2]
 
   # Extra check: 'ceiling' outliers must be on COLS and C_LP lines
@@ -237,26 +247,28 @@ p_get_outlier <- function (data, combo, params, k) {
 p_get_values <- function (data.new, params) {
   model.new <- nca_analysis(data.new, params$x, params$y,
                             ceilings = params$ceiling, corner = params$corner,
-                            flip.x = params$flip.x, flip.y = params$flip.y)
+                            flip.x = params$flip.x, flip.y = params$flip.y,
+                            scope = params$scope)
   eff.nw <- model.new$summaries[[1]]$params[2]
   dif.abs <- ifelse(is.na(eff.nw), 0, eff.nw - params$eff.or)
   dif.rel <- ifelse(params$eff.or < epsilon, 0, 100 * dif.abs / params$eff.or)
-  return(c(eff.nw, dif.abs, dif.rel))
+  global.new <- model.new$summaries[[1]]$global
+  return(list(eff.nw, dif.abs, dif.rel, global.new))
 }
 
 
-p_zone_scope <- function (data, combo, params) {
-  found_scope <- any(data[combo, params$x] %in% params$global[c(3, 4)]) ||
-    any(data[combo, params$y] %in% params$global[c(5, 6)])
+p_zone_scope <- function (combo, params, global.new) {
+  idx <- ncol(global.new)
+  found_scope <- !all(global.new[2:6, idx] == params$global[2:6, idx])
 
   if (params$ceiling[1] %in% p_no_peer_line) {
-    found_zone <- T
+    found_ceiling <- T
   }
   else {
-    found_zone <- any(combo %in% rownames(params$peers))
+    found_ceiling <- any(combo %in% rownames(params$peers))
   }
 
-  return(c(ifelse(found_zone, "X", ""), ifelse(found_scope, "X", "")))
+  return(c(ifelse(found_ceiling, "X", ""), ifelse(found_scope, "X", "")))
 }
 
 
@@ -272,27 +284,65 @@ p_get_names <- function (combo, k) {
 }
 
 
-p_format_outliers <- function (outliers, max.results, k) {
+p_format_outliers <- function (outliers, max.results, k, min.dif, condensed) {
   # The dataset / param combination doesn't produce outliers
   if (nrow(outliers) == 0) {
     return()
   }
 
   outliers <- data.frame(outliers)
-  outliers <- outliers[, names(outliers) != "combo"]
-  for (i in 1:length(outliers)) { outliers[[i]] <- unlist(outliers[[i]]) }
-  outliers <- outliers[order(-abs(outliers$dif.rel)),]
+
+  for (i in 1:7) { outliers[[i]] <- unlist(outliers[[i]]) }
+  for (row_idx in 1:nrow(outliers)) {
+    len <- length(unlist(strsplit(outliers[row_idx, 1], ' - ', fixed=T)))
+    outliers[row_idx, 9] <- len
+  }
+  outliers <- outliers[order(-abs(outliers$dif.rel), outliers[, 9]),]
+  org.length <- nrow(outliers)
+
+  # If k == 1: all outliers are shown on Plotly
+  if (k != 1) {
+    # Strip combos where extra observations do not change outcome
+    if (nrow(outliers) > 1 && condensed) {
+      keep <- 1
+      # Remove outliers which are not larger than previous (but keep singles)
+      for (row_idx in 2:nrow(outliers)) {
+        parts <- unlist(strsplit(outliers[row_idx, 1], ' - ', fixed=T))
+        current <- abs(outliers[row_idx, 5])
+        prev <- abs(outliers[row_idx - 1, 5])
+        if (length(parts) == 1 || abs(current - prev) > min.dif) {
+          keep <- c(keep, row_idx)
+        }
+      }
+      outliers <- outliers[keep, ]
+    }
+
+    outliers <- outliers[1:min(nrow(outliers), max.results),]
+  }
+
+  outliers <- outliers[, -c(8, 9)]
   outliers[, c(2, 3, 4)] <- round(outliers[, c(2, 3, 4)], digits = 2)
   outliers[, 5] <- round(outliers[, 5], digits = 1)
 
-  if (k != 1) {
-    # If k == 1: all outliers are shown on Plotly
-    outliers <- outliers[1:min(nrow(outliers), max.results),]
-    fmt <- paste0("%- ", max(nchar(outliers$outliers)), "s")
-    outliers$outliers <- sprintf(fmt, trimws(outliers$outliers, which = "left"))
+  if (org.length > nrow(outliers)) {
+    attr(outliers, SHOWN) <- nrow(outliers)
+    attr(outliers, HIDDEN) <- org.length - nrow(outliers)
   }
+  class(outliers) <- c("outliers", class(outliers))
 
   rownames(outliers) <- NULL
-
   return(outliers)
 }
+
+
+print.outliers <- function (x, ...) {
+  # NextMethod()
+  print(format(x, justify = "left"))
+
+  hidden <- attr(x, HIDDEN)
+  shown <- attr(x, SHOWN)
+  # Only show when 'full' outliers, not when selection
+  if (!is.null(hidden) && shown == nrow(x)) {
+    message(sprintf("# Not showing %s possible outliers", hidden))
+  }
+}
